##-----------------------------LICENSE NOTICE------------------------------------
##  This file is part of CPCtelera: An Amstrad CPC Game Engine 
##  Copyright (C) 2018 ronaldo / Fremos / Cheesetea / ByteRealms (@FranGallegoBR)
##
##  This program is free software: you can redistribute it and/or modify
##  it under the terms of the GNU Lesser General Public License as published by
##  the Free Software Foundation, either version 3 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU Lesser General Public License for more details.
##
##  You should have received a copy of the GNU Lesser General Public License
##  along with this program.  If not, see <http://www.gnu.org/licenses/>.
##------------------------------------------------------------------------------
############################################################################
##                        CPCTELERA ENGINE                                ##
##                 Automatic compression utilities                        ##
##------------------------------------------------------------------------##
## This file is intended for users to automate the generation of          ##
## compressed files and their inclusion in users' projects.               ##
############################################################################

## COMPRESSION EXAMPLE (Uncomment lines to use)
##

## First 3 calls to ADD2PACK add enemy, hero and background
## graphics (previously converted to binary data) into the 
## compressed group 'mygraphics'. After that, call to PACKZX7B
## compresses all the data and generates an array with the result
## that is placed in src/mygraphics.c & src/mygraphics.h, ready
## to be included and used by other modules.
##
#$(eval $(call ADD2PACK,mygraphics,gfx/enemy.bin))
#$(eval $(call ADD2PACK,mygraphics,gfx/hero.bin))
#$(eval $(call ADD2PACK,mygraphics,gfx/background.bin))
#$(eval $(call PACKZX7B,mygraphics,src/))




############################################################################
##              DETAILED INSTRUCTIONS AND PARAMETERS                      ##
##------------------------------------------------------------------------##
##                                                                        ##
## Macros used for compression are ADD2PACK and PACKZX7B:                 ##
##                                                                        ##
##	ADD2PACK: Adds files to packed (compressed) groups. Each call to this ##
##  		  macro will add a file to a named compressed group.          ##
##  PACKZX7B: Compresses all files in a group into a single binary and    ##
##            generates a C-array and a header to comfortably use it from ##
##            inside your code.                                           ##
##                                                                        ##
##------------------------------------------------------------------------##
##                                                                        ##
##  $(eval $(call ADD2PACK,<packname>,<file>))                            ##
##                                                                        ##
##		Sequentially adds <file> to compressed group <packname>. Each     ##
## call to this macro adds a new <file> after the latest one added.       ##
## packname could be any valid C identifier.                              ##
##                                                                        ##
##  Parameters:                                                           ##
##  (packname): Name of the compressed group where the file will be added ##
##  (file)    : File to be added at the end of the compressed group       ##
##                                                                        ##
##------------------------------------------------------------------------##
##                                                                        ##
##  $(eval $(call PACKZX7B,<packname>,<dest_path>))                       ##
##                                                                        ##
##		Compresses all files in the <packname> group using ZX7B algorithm ##
## and generates 2 files: <packname>.c and <packname>.h that contain a    ##
## C-array with the compressed data and a header file for declarations.   ##
## Generated files are moved to the folder <dest_path>.                   ##
##                                                                        ##
##  Parameters:                                                           ##
##  (packname) : Name of the compressed group to use for packing          ##
##  (dest_path): Destination path for generated output files              ##
##                                                                        ##
############################################################################
##                                                                        ##
## Important:                                                             ##
##  * Do NOT separate macro parameters with spaces, blanks or other chars.##
##    ANY character you put into a macro parameter will be passed to the  ##
##    macro. Therefore ...,src/sprites,... will represent "src/sprites"   ##
##    folder, whereas ...,  src/sprites,... means "  src/sprites" folder. ##
##  * You can omit parameters by leaving them empty.                      ##
##  * Parameters (4) and (5) are optional and generally not required.     ##
############################################################################

$(eval $(call ADD2PACK,nivel3,src/tilemaps/Bos_lvl3.bin))
$(eval $(call PACKZX7B,nivel3,src/tilemaps/))
$(eval $(call ADD2PACK,nivel2,src/tilemaps/Bos_lvl2.bin))
$(eval $(call PACKZX7B,nivel2,src/tilemaps/))
$(eval $(call ADD2PACK,nivel1,src/tilemaps/Bos_lvl1.bin))
$(eval $(call PACKZX7B,nivel1,src/tilemaps/))
$(eval $(call ADD2PACK,nivel6,src/tilemaps/Fab_lvl3.bin))
$(eval $(call PACKZX7B,nivel6,src/tilemaps/))
$(eval $(call ADD2PACK,nivel5,src/tilemaps/Fab_lvl2.bin))
$(eval $(call PACKZX7B,nivel5,src/tilemaps/))
$(eval $(call ADD2PACK,nivel4,src/tilemaps/Fab_lvl1.bin))
$(eval $(call PACKZX7B,nivel4,src/tilemaps/))
$(eval $(call ADD2PACK,nivel9,src/tilemaps/CH_lvl3.bin))
$(eval $(call PACKZX7B,nivel9,src/tilemaps/))
$(eval $(call ADD2PACK,nivel8,src/tilemaps/CH_lvl2.bin))
$(eval $(call PACKZX7B,nivel8,src/tilemaps/))
$(eval $(call ADD2PACK,nivel7,src/tilemaps/CH_lvl1.bin))
$(eval $(call PACKZX7B,nivel7,src/tilemaps/))
$(eval $(call ADD2PACK,arco,src/tilemaps/arco_test.bin))
$(eval $(call PACKZX7B,arco,src/tilemaps/))
$(eval $(call ADD2PACK,boss,src/tilemaps/Boss_lvl.bin))
$(eval $(call PACKZX7B,boss,src/tilemaps/))

# Compresion pantallas
#$(eval $(call ADD2PACK,menu_0,src/sprites/menu_0.bin))
#$(eval $(call PACKZX7B,menu_0,src/pantallas/))
#
#$(eval $(call ADD2PACK,menu_1,src/sprites/menu_1.bin))
#$(eval $(call PACKZX7B,menu_1,src/pantallas/))
#
#$(eval $(call ADD2PACK,menu_2,src/sprites/menu_2.bin))
#$(eval $(call PACKZX7B,menu_2,src/pantallas/))
#
#$(eval $(call ADD2PACK,menu_3,src/sprites/menu_3.bin))
#$(eval $(call PACKZX7B,menu_3,src/pantallas/))
#
#$(eval $(call ADD2PACK,menu_4,src/sprites/menu_4.bin))
#$(eval $(call PACKZX7B,menu_4,src/pantallas/))
#
#$(eval $(call ADD2PACK,menu_5,src/sprites/menu_5.bin))
#$(eval $(call PACKZX7B,menu_5,src/pantallas/))

$(eval $(call ADD2PACK,menu_final,src/sprites/Menu.bin))
$(eval $(call PACKZX7B,menu_final,src/pantallas/))

$(eval $(call ADD2PACK,game_mode,src/sprites/Game_mode.bin))
$(eval $(call PACKZX7B,game_mode,src/pantallas/))

#$(eval $(call ADD2PACK,escenas_0,src/sprites/escenas_0.bin))
#$(eval $(call PACKZX7B,escenas_0,src/pantallas/))#bosque
#
#$(eval $(call ADD2PACK,escenas_1,src/sprites/escenas_1.bin))
#$(eval $(call PACKZX7B,escenas_1,src/pantallas/))
#
#$(eval $(call ADD2PACK,escenas_2,src/sprites/escenas_2.bin))
#$(eval $(call PACKZX7B,escenas_2,src/pantallas/))
#
#$(eval $(call ADD2PACK,escenas_3,src/sprites/escenas_3.bin))
#$(eval $(call PACKZX7B,escenas_3,src/pantallas/))#boss
